#!/bin/bash

#----
##
## This set of funtions are used to perform unit tests on bash functions.
##
## @Globals INFO the file or device to send informational messages to
## @Globals ERR_FILE file which records failed tests, used for counting the number of errors later
## @Globals ERR the file or device to send critical messages and debugging information
## @Globals OK_FILE File which records successful tests
## @Globals SUT_TEST_DIR temporary directory for tests
## @Globals TEST_OK Value a function should return if the test succeeded
## @Globals TEST_FAIL Value a function should retuirn if the test failed
## @Globals TEST_WARNING Unused
#----

#-----
## Creates empty testing directory and puts it's name to stdout
## @Globals SUT_TEST_DIR
#-----
function sut_create_temp_dir() {
	rm -rf "$SUT_TEST_DIR" && mkdir -p "$SUT_TEST_DIR" ||
		sut_error "Can't create test directory '$SUT_TEST_DIR'"
	echo $SUT_TEST_DIR
}

#-----
## Puts name of the testing directory to stdout
## @Globals SUT_TEST_DIR
#-----
function sut_get_temp_dir() {
	echo $SUT_TEST_DIR
}

#----
## Function called when a test fails. This should generaly only be called 
## by the <@sut_test> function.
## @Globals INFO
## @Globals ERR_FILE
## @Args error message
#----
function sut_error() {
	echo "ERROR: " "$@" >>$INFO
	echo "$@" >>$ERR_FILE
}

#----
## Function called when a test succeeds. This should generaly only be called
## by the <@sut_test> function.
## @Globals INFO
## @Globals OK_FILE
## @Args Success message
#----
function sut_ok() {
	echo "OK: " "$@" >>$INFO
	echo "$@" >>$OK_FILE
}

#----
## Used to send debugging messages to the user. Should only be used from
## DEBUG_MESSAGE. ex:<code>
## eval $DEBUG_MESSAGE "This is a debugging message"
## </code>
## @Globals ERR
## @param Function message was called from
## @param Line message was called on
## @param Debug message
#----
function sut_debug_message() {
	echo -n "DEBUG: Function $1, line $2: " >>$ERR
	shift 2
	echo "$@" >>$ERR
}
# Use like: eval $DEBUG_MESSAGE "some message"
DEBUG_MESSAGE='sut_debug_message "$FUNCNAME" "$LINENO"'

#----
## Clears the files for error and success counts
#----
function sut_reset_counts() {
	[ -f $ERR_FILE ] && rm $ERR_FILE 
	[ -f $OK_FILE ] && rm $OK_FILE
}

#----
## Report the success and fail counts for tests
## @return 0 if there wereno errors
## @return 1 if there were errors
#----
function sut_count_report() {
	local ok=$(wc -l $OK_FILE 2>/dev/null | awk '{print $1;}')
	local err=$(wc -l $ERR_FILE 2>/dev/null | awk '{print $1;}')
	let ok=ok
	let err=err
	local total
	let total=ok+err

	echo "* $1 results"

	echo -en "* "
	[ $ok -lt $total ] && echo -en "\e[1m"
	echo -e "Report: OK  ${ok}/${total}\e[0m" >>$INFO
	
	echo -en "* "
	[ $err -gt 0 ] && echo -en "\e[1m"
	echo -e "\e[31mReport: ERR ${err}/${total}\e[0m" >>$INFO
	
	echo
	
	[ ${err} -eq 0 ]
}

#----
## Sources the file but only allows the specified functions to be defined 
## @param File to source
## @param Function to allow out
## @param (optional) Function to allow out
## @param (optional) ...
#----
function sut_isolate_functions() {
#	eval $DEBUG_MESSAGE "isolating $*"
	local FILE=$1
	shift
	eval "$( source $FILE;
		for func in $* ; do
			if ! declare -f $func ; then
				sut_error "$func not found in $FILE"
			fi
		done
	)"
}

#----
## Sources the file and removes the specified functions from being defined
## @param File to source
## @param Function to allow out
## @param (optional) Function to allow out
## @param (optional) ...
#----
function sut_ignore_functions() {
#	eval $DEBUG_MESSAGE "ignoring $*"
	local FILE=$1
	shift

	good_functions=$( ( declare -F 
		source $*
		for i in $* ; do
			unset $i
		done
		declare -F ) | sort | uniq -u | sed 's/^declare -f //' )
	sut_isolate_functions $FILE $good_functions
	
}

#----
## Lists all functions defined in the source. 
## Warning, like the above functions, this sources the file. This may cause
## BadThings to happen if it is actualy a script and not just a pure lib.
## @param File to source
#----
function sut_functions_in() {
	( declare -F ; source $* ; declare -F ) |
		sort | uniq -u | sed 's/^declare -f //'
}

#----
## Tests a module. Each module should have a TESTS variable
## declared at the top listing the groups to test.
## @param File to test functions of
## @param File containing SUT module tests.
#----
function sut_test_module() { (
	local src=$1
	local tests=$2
	local err_count=0
	source $tests
	for i in $TESTS ; do
		sut_test_group ${i}_gtest $src
		let err_count+=$?
	done
	return $err_count
	#TODO perhaps list all functions not checked
) }

#----
## Tests a single group test from a module.
## Reports on theresults and clears the reoprt counts.
#----
function sut_test_group() {
	local err
	$1 $src
	sut_count_report $1
	err=$?
	sut_reset_counts
	return $err
}

#----
## Performs a single test and interprets the return value.
#----
function sut_test() {
	local NAME="$1"
	shift
        (
	       "${@}" && sut_ok "$NAME" && return 0 || sut_error "$NAME"
	       return 1
        )
        return $?
}

ERR_FILE="/tmp/SUT.errors"
OK_FILE="/tmp/SUT.ok"
SUT_TEST_DIR="/tmp/SUT.test.dir"

TEST_OK=0
TEST_FAIL=1
TEST_WARNING=2

#sut_reset_counts
#sut_test_module $1 $2
#isolate_functions "$@"
#count_report
